/*******************************************************************************************************************
Copyright (c) 2012 Cycling '74

Permission is hereby granted, free of charge, to any person obtaining a copy of this software
and associated documentation files (the "Software"), to deal in the Software without restriction,
including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense,
and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so,
subject to the following conditions:

The above copyright notice and this permission notice shall be included in all copies
or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE
OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
*******************************************************************************************************************/

#include "MyVSTPlugin.h"

#define UNIQUE_ID 'Gen_'
#define VST_NAME "Gen VST"
#define VST_VENDOR "Cycling '74 Example"
#define VST_VENDOR_VERSION 1000

#ifndef MIN
#define MIN(a, b) (((a) < (b)) ? (a) : (b))
#endif

void FloatToSample(t_sample *dst, const float *src, long n) {
	while (n--) *dst++ = *src++;
}

void FloatFromSample(float *dst, const t_sample *src, long n) {
	while (n--) *dst++ = *src++;
}

void zeroBuffer(t_sample *dst, int n) {
	while (n--) *dst++ = 0;
}

//-------------------------------------------------------------------------------------------------------
AudioEffect* createEffectInstance (audioMasterCallback audioMaster)
{
	return new MyVSTPlugin(audioMaster);
}

//-------------------------------------------------------------------------------------------------------
MyVSTPlugin::MyVSTPlugin (audioMasterCallback audioMaster)
: AudioEffectX (audioMaster, 1, num_params())	// 1 program, N parameters
, gen(0)
, bufferSize(512)
, sampleRate(44100)
, blockSize(512)
{
	inputBuffers.resize(num_inputs());
	outputBuffers.resize(num_outputs());

	for(int i=0; i < inputBuffers.size(); i++) {
		inputBuffers[i] = new t_sample[bufferSize];
	}
	for(int i=0; i < outputBuffers.size(); i++) {
		outputBuffers[i] = new t_sample[bufferSize];
	}

	gen = (CommonState *)create(sampleRate, blockSize);
	setNumInputs (num_inputs());
	setNumOutputs (num_outputs());
	setUniqueID (UNIQUE_ID);	// identify
	canProcessReplacing ();	// supports replacing output
	canDoubleReplacing ();	// supports double precision processing

	vst_strncpy (programName, VST_NAME, kVstMaxProgNameLen);	// default program name
}

//-------------------------------------------------------------------------------------------------------
MyVSTPlugin::~MyVSTPlugin ()
{
	for(int i=0; i < inputBuffers.size(); i++) {
		delete[] inputBuffers[i];
	}
	for(int i=0; i < outputBuffers.size(); i++) {
		delete[] outputBuffers[i];
	}
	destroy(gen);
	
}

//-------------------------------------------------------------------------------------------------------
void MyVSTPlugin::setProgramName (char* name)
{
	vst_strncpy (programName, name, kVstMaxProgNameLen);
}

//-----------------------------------------------------------------------------------------
void MyVSTPlugin::getProgramName (char* name)
{
	vst_strncpy (name, programName, kVstMaxProgNameLen);
}

//-----------------------------------------------------------------------------------------
void MyVSTPlugin::setSampleRate(float v)
{
	sampleRate = v;
	gen->sr = v;
	reset(gen);
}

//-----------------------------------------------------------------------------------------
void MyVSTPlugin::setBlockSize(VstInt32 v)
{
	blockSize = v;
	gen->vs = v;
	reset(gen);
}

//-----------------------------------------------------------------------------------------
void MyVSTPlugin::setParameter (VstInt32 index, float value)
{
	setparameter(gen, index, value, 0);
}

//-----------------------------------------------------------------------------------------
float MyVSTPlugin::getParameter (VstInt32 index)
{
	double val;
	getparameter(gen, index, &val);
	return (float)val;
}

//-----------------------------------------------------------------------------------------
void MyVSTPlugin::getParameterName (VstInt32 index, char* label)
{
	ParamInfo *param = gen->params+index;
	vst_strncpy (label, param->name, kVstMaxParamStrLen);
}

//-----------------------------------------------------------------------------------------
void MyVSTPlugin::getParameterDisplay (VstInt32 index, char* text)
{
	double val;
	getparameter(gen, index, &val);
	float2string((float)val, text, kVstMaxParamStrLen);
}

//-----------------------------------------------------------------------------------------
void MyVSTPlugin::getParameterLabel (VstInt32 index, char* label)
{
	vst_strncpy (label, "", kVstMaxParamStrLen);
}

//------------------------------------------------------------------------
bool MyVSTPlugin::getEffectName (char* name)
{
	vst_strncpy (name, VST_NAME, kVstMaxEffectNameLen);
	return true;
}

//------------------------------------------------------------------------
bool MyVSTPlugin::getProductString (char* text)
{
	vst_strncpy (text, VST_NAME, kVstMaxProductStrLen);
	return true;
}

//------------------------------------------------------------------------
bool MyVSTPlugin::getVendorString (char* text)
{
	vst_strncpy (text, VST_VENDOR, kVstMaxVendorStrLen);
	return true;
}

//-----------------------------------------------------------------------------------------
VstInt32 MyVSTPlugin::getVendorVersion ()
{ 
	return VST_VENDOR_VERSION; 
}

//-----------------------------------------------------------------------------------------
void MyVSTPlugin::processReplacing (float** inputs, float** outputs, VstInt32 sampleFrames)
{
	int nin = (int)inputBuffers.size();
	int nout = (int)outputBuffers.size();
	int n = 0;
	while(n < sampleFrames) {
		int toprocess = MIN(bufferSize, (sampleFrames-n));
			
		for(int i=0; i < nin; i++) {
			FloatToSample(inputBuffers[i], inputs[i]+n, toprocess);
		}
		for(int i=nin; i < inputBuffers.size(); i++) {
			zeroBuffer(inputBuffers[i], toprocess);
		}
			
		perform(
			gen,
			&(inputBuffers[0]), inputBuffers.size(),
			&(outputBuffers[0]), outputBuffers.size(),
			toprocess
		);
			
		for(int i=0; i < nout; i++) {
			FloatFromSample(outputs[i]+n, outputBuffers[i], toprocess);
		}

		n += toprocess;
	}
}

//-----------------------------------------------------------------------------------------
void MyVSTPlugin::processDoubleReplacing (double** inputs, double** outputs, VstInt32 sampleFrames)
{
    perform(gen, inputs, num_inputs(), outputs, num_outputs(), sampleFrames);
}
